/** *****************************************************
 * Copyright (C) HRider in motion S.L.
 *
 * This file is part of HRider.
 *
 * HRider can not be copied and/or distributed without the express
 * permission of HRider in motion S.L.
 ****************************************************** */
package net.hrider.api.usecases;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.hrider.api.client.ApiException;
import net.hrider.api.model.Appraisal;
import net.hrider.api.model.AppraisalSummary;
import net.hrider.api.model.Company;
import net.hrider.api.model.EmployeeSummary;
import net.hrider.api.model.FeedbackReport;
import net.hrider.api.model.Link;

/**
 * Example to get the feedback report of an employee in an appraisal.
 * 
 * @author ahc (aina@hrider.net)
 */
public class GetPersonalReport extends AbstractUseCase {

    private final long companyId;
    private final long appraisalId;
    private final long employeeId;
    private final String outputDir;

    private static final Logger LOG = Logger.getLogger(GetPersonalReport.class.getName());

    public GetPersonalReport(long companyId, long appraisalId, long employeeId, String outputDir) {
        this.companyId = companyId;
        this.appraisalId = appraisalId;
        this.employeeId = employeeId;
        this.outputDir = outputDir;
    }
    
    @Override
    public void doRun() throws ApiException {
        // Find company
        Link companyLink = root.getLinks().getCompanies().build("" + companyId);
        Company company = service.get(Company.class, companyLink);

        // Get appraisal
        Link appraisalLink = company.getLinks().getAppraisals().build("" + appraisalId);
        Appraisal appraisal = service.get(Appraisal.class, appraisalLink);

        // Get report
        Link reportLink = appraisal.getLinks().getReports().build("feedback-reports").build("" + employeeId);
        FeedbackReport report = service.get(FeedbackReport.class, reportLink);
        printFeedbackReport(report);
        
        // Get PDF
        InputStream in = service.get(InputStream.class, reportLink, net.hrider.api.Constants.MIMETYPE_PDF);
        // alternative:
        //InputStream in = service.get(InputStream.class, reportLink.addQueryParam("alt", "application/pdf"));
        try (FileOutputStream fout = new FileOutputStream(new File(outputDir, "feedback-report.pdf"))) {
            saveToFile(in, fout);
        } catch (IOException ex) {
            LOG.log(Level.SEVERE, "error saving file", ex);
        }        
    }

    /**
     * Main method. Accepts 4 arguments, the company ID, the appraisal ID,
     * the employee ID and the output directory.
     * 
     * @param args company ID (long), appraisal ID (long) and employee ID (long).
     */
    public static void main(String[] args) {
        if (args.length == 4) {
            try {
                GetPersonalReport uc = new GetPersonalReport(Long.parseLong(args[0]),
                        Long.parseLong(args[1]), Long.parseLong(args[2]), args[3]);
                uc.run();
            } catch (NumberFormatException ex) {
                printUsage();
            }
        } else {
            printUsage();
        }
    }

    // private methods *********************************************************
    
    private static void printUsage() {
        System.err.println("Usage: companyId appraisalId employeeId");
        System.err.println("companyId: the company id (long). \n"
                + "appraisalId: the appraisal id (long). \n"
                + "employeeId: the employee id (long). \n" 
                + "outputDir: path to the output directory to save the file.");
    }
    
    private void printFeedbackReport(FeedbackReport report) {
        EmployeeSummary employee = report.getEmbedded().getEmployee();
        AppraisalSummary appraisal = report.getEmbedded().getAppraisal();
        // print tabular data
        StringBuilder out = new StringBuilder();
        String format = "%-20s%-20s%-20s%-23s%-10s%-10s%-10s";
        out.append(String.format(format, "Employee", "Appraisal", "Position", "% Achieved Indicators", "Score", "Gap", "Grade")).append("\n");
        out.append(String.format(format, "-------------------", "-------------------", "-------------------", "----------------------", "---------", "---------", "---------")).append("\n");
        out.append(String.format(format, employee.getCode(), appraisal.getCode(), 
                report.getPositionCode(), report.getPercentageAchievedIndicators(),
                report.getScore(), report.getGap(), report.getGrade())).append("\n");
        System.out.println(out);
    }
}
